package com.sf.vertx.utils;

import java.util.List;
import java.util.regex.Pattern;
import static com.sf.vertx.constans.SACConstants.HTTPS;
import org.apache.commons.lang3.StringUtils;

import com.sf.vertx.api.pojo.Node;
import com.sf.vertx.api.pojo.RouteContent;
import com.sf.vertx.arithmetic.roundRobin.SacLoadBalancing;
import com.sf.vertx.arithmetic.roundRobin.WeightedRoundRobin;
import com.sf.vertx.handle.AppConfigHandler;
import com.sf.vertx.util.SacSocketAddress;

import io.vertx.core.http.HttpServerRequest;
import io.vertx.core.net.SocketAddress;
import io.vertx.core.net.impl.SocketAddressImpl;
import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.HttpException;
import lombok.extern.slf4j.Slf4j;

/***
 * 反向代理工具类
 * 
 * @author xy
 *
 */
@Slf4j
public class ProxyTool {

	public static SocketAddress resolveOriginAddress(HttpServerRequest request) {
		String appCode = request.getHeader(AppConfigHandler.getAppCodeHeaderKey());
		String apiCode = request.getHeader(AppConfigHandler.getApiCodeHeaderKey());
		log.info("uri:{}, header appCode:{},apiCode:{}", request.uri(), appCode, apiCode);
		return resolveOriginAddress(request, appCode, apiCode);
	}

	public static SocketAddress resolveOriginAddress(RoutingContext ctx) {
		String appCode = AppUtils.getAppConfigFromRoutingContext(ctx).getAppCode();
		String apiCode = AppUtils.getApiConfigFromRoutingContext(ctx).getApiCode();
		log.info("uri:{}, header appCode:{},apiCode:{}", ctx.request().uri(), appCode, apiCode);
		return resolveOriginAddress(ctx.request(), appCode, apiCode);
	}

	private static SocketAddress resolveOriginAddress(HttpServerRequest request, String appCode, String apiCode) {
		// 判断 "routeType": "WEIGHT_ROUTE", // 路由类型 WEIGHT_ROUTE ，HEADER_ROUTE
		String key = appCode + ":" + apiCode;
		Integer routerType = AppConfigHandler.routerType(key);
		//SocketAddress socketAddress = null;
		SacSocketAddress socketAddress = null;
		switch (routerType) {
		case 1:
			SacLoadBalancing sacLoadBalancing = AppConfigHandler.getLoadBalancing(key);
			Node node = sacLoadBalancing.selectNode();
			//socketAddress = SocketAddress.inetSocketAddress(node.getPort(), node.getIp());
			socketAddress = new SacSocketAddress(node.getPort(), node.getIp()); 
			socketAddress.setSSL(StringUtils.equals(node.getProtocol(), HTTPS));
			log.info("sacLoadBalancing address:{},port:{}", socketAddress.host(), socketAddress.port());
			return socketAddress;
		case 2:
			List<RouteContent> routeContentList = AppConfigHandler.routerHeaderConentList(key);
			if (routeContentList != null && routeContentList.size() > 0) {
				for (RouteContent routeContent : routeContentList) {
					String headerValue = request.getHeader(routeContent.getHeaderKey());
					List<String> headerValues = routeContent.getHeaderValues();
					//String matchType = routeContent.getMatchType();
					if (headerValues.contains(headerValue)) {
//						socketAddress = SocketAddress.inetSocketAddress(routeContent.getServerAddress().getPort(),
//								routeContent.getServerAddress().getHost());
						socketAddress = new SacSocketAddress(routeContent.getServerAddress().getPort(),
								routeContent.getServerAddress().getHost()); 
						socketAddress.setSSL(StringUtils.equals(routeContent.getServerAddress().getProtocol(), HTTPS));
						log.info("sacLoadBalancing address:{},port:{}", socketAddress.host(), socketAddress.port());
						return socketAddress;
					}
				}
			}
			break;
		}

		// 抛出异常,无法找到负载均衡node节点
		throw new HttpException(10021);
	}

	public static boolean regexMatch(String pattern, String target) {
		return Pattern.matches(pattern, target);
	}

	public static SacLoadBalancing roundRobin(List<Node> nodeList) {
		WeightedRoundRobin weightedRoundRobin = new WeightedRoundRobin();
		for (Node node : nodeList) {
			int weight = node.getWeight() != null ? node.getWeight() : 1;
			node.setWeight(weight);
			node.setCurrentWeight(weight);
			node.setEffectiveWeight(weight);
			weightedRoundRobin.totalWeight += node.getEffectiveWeight();
		}
		weightedRoundRobin.init(nodeList);
		return weightedRoundRobin;
	}
}
